# Emmy version 1.0.0
# EmmyObserver.py
# by Antoine Rauzy
# Copyrights (c) 2019 AltaRica Association
#
# This file is part of Emmy, a pedagogical stochastic simulator.
# Emmy is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# Emmy is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with Emmy.  If not, see <http://www.gnu.org/licenses/>.

# Table of Contents
# -----------------
# 1) Imported modules
# 2) Observers

# 1) Imported modules
# -------------------

from S2MLCore import *

from EmmyType import *

# 2) Observers
# ------------

class EmmyObserver(S2MLPort):
  def __init__(self, domain, path, definition, container):
    S2MLPort.__init__(self, EmmyType.OBSERVER, domain, path, container)
    self.SetDefinition(definition)
    self._index = 0
    self._indicatorProfile = None
    self._indicators = []
    self._value = S2MLConstant.NewNone()
    self._fdsObserver = None

  def GetIndex(self):
    return self._index
  def SetIndex(self, index):
    self._index = index

  def IsValueCalculated(self):
    return not self._value.IsNone()
  def ResetValue(self):
    self._value.Reset()
  def GetValue(self):
    return self._value
  def SetValue(self, value):
    self._value.Set(value)

  def GetIndicatorProfile(self):
    return self._indicatorProfile
  def SetIndicatorProfile(self, profile):
    self._indicatorProfile = profile

  def CreateIndicators(self, numberOfIndicators):
    domain = self.GetDomain()
    self._indicators = [S2MLIndicator(self._indicatorProfile, domain) for i in range(0, numberOfIndicators)]
  def GetNumberOfIndicators(self):
    return len(self._indicators)
  def GetIndicators(self):
    return self._indicators
  def GetIndicator(self, index):
    if index<0 or index>=self.GetNumberOfIndicators():
      return None
    return self._indicators[index]
  def RecordValueInIndicator(self, index):
    if index<0 or index>=self.GetNumberOfIndicators():
      return
    indicator = self._indicators[index]
    indicator.AddValue(self._value)

  def GetFDSObserver(self):
    return self._fdsObserver
  def SetFDSObserver(self, fdsObserver):
    self._fdsObserver = fdsObserver
