# S2MLCore version 1.0.0
# S2MLMeasure.py
# by Antoine Rauzy
# Copyrights (c) 2018 AltaRica Association
#
# This file is part of S2MLCore.
# S2MLCore is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# S2MLCore is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with S2MLCore.  If not, see <http://www.gnu.org/licenses/>.

# Table of Contents
# -----------------
# 1) Imported modules
# 2) Measures

# 1) Imported modules
# -------------------

from S2MLDomains import *

from S2MLBin import *

# 2) Measures
# -----------

class S2MLMeasure:
  MAXIMUM_NUMBER_OF_BINS = 100
  DEFAULT_NUMBER_OF_BINS = 0
  MAXIMUM_NUMBER_OF_STORED_VALUES = 10000
  DEFAULT_NUMBER_OF_STORED_VALUES = 1000
  MINIMUM_RATIO_NUMBER_OF_STORED_VALUES_NUMBER_OF_BINS = 10

  def __init__(self, numberOfBins = DEFAULT_NUMBER_OF_BINS, numberOfStoredValues = DEFAULT_NUMBER_OF_STORED_VALUES):
    self._identifier = None
    self._bin = S2MLBin()
    if numberOfBins>S2MLMeasure.MAXIMUM_NUMBER_OF_BINS:
      self._numberOfBins = S2MLMeasure.MAXIMUM_NUMBER_OF_BINS
    else:
      self._numberOfBins = numberOfBins
    if numberOfStoredValues<self._numberOfBins*S2MLMeasure.MINIMUM_RATIO_NUMBER_OF_STORED_VALUES_NUMBER_OF_BINS:
      self._numberOfStoredValues = self._numberOfBins*S2MLMeasure.MINIMUM_RATIO_NUMBER_OF_STORED_VALUES_NUMBER_OF_BINS
    else:
      self._numberOfStoredValues = numberOfStoredValues
    if self._numberOfStoredValues>S2MLMeasure.MAXIMUM_NUMBER_OF_STORED_VALUES:
      self._numberOfStoredValues = S2MLMeasure.MAXIMUM_NUMBER_OF_STORED_VALUES
    if self._numberOfBins==0:
      self._bins = []
    else:
      self._bins = [S2MLBin() for i in range(0, self._numberOfBins)]
    if self._numberOfStoredValues==0:
      self._storedValues = []
    else:
      self._storedValues = [0.0 for i in range(0, self._numberOfStoredValues)]
    self._storedValuesIndex = 0

  def Reset(self):
    self._bin.Reset()
    for bin in self._bins:
      bin.Reset()
    for i in range(0, self._numberOfStoredValues):
      self._storedValues[i] = 0.0
    self._storedValuesIndex = 0

  def GetBin(self):
    return self._bin

  def GetNumberOfValues(self):
    return self._bin.GetNumberOfValues()
  def GetMinimumValue(self):
    return self._bin.GetMinimumValue()
  def GetMaximumValue(self):
    return self._bin.GetMaximumValue()

  def GetMean(self):
    return self._bin.GetMean()
  def GetVariance(self):
    return self._bin.GetVariance()
  def GetStandardDeviation(self):
    return self._bin.GetStandardDeviation()

  def GetConfidenceSlack90(self):
    return self._bin.GetConfidenceSlack90()
  def GetConfidenceSlack95(self):
    return self._bin.GetConfidenceSlack95()
  def GetConfidenceSlack99(self):
    return self._bin.GetConfidenceSlack99()

  def GetErrorFactor90(self):
    return self._bin.GetErrorFactor90()
  def GetErrorFactor95(self):
    return self._bin.GetErrorFactor95()
  def GetErrorFactor99(self):
    return self._bin.GetErrorFactor99()

  def GetNumberOfBins(self):
    return self._numberOfBins
  def GetNthBin(self, index):
    if index<0 or index>=self._numberOfBins:
      return None
    return self._bins[index]

  def GetStoredValuesIndex(self):
    return self._storedValuesIndex
  def GetStoredValue(self, index):
    if index<0 or index>=self._storedValuesIndex:
      return 0.0
    return self._storedValues[index]

  def AddValue(self, value):
    self._bin.AddValue(value)
    if self._numberOfStoredValues==0:
      return
    if self._storedValuesIndex==self._numberOfStoredValues:
      self.TransferStoredValuesIntoBins()
    self._storedValues[self._storedValuesIndex] = value
    self._storedValuesIndex += 1

  def TransferStoredValuesIntoBins(self):
    if self._storedValuesIndex==0:
      return
    if self._numberOfBins==0:
      self._storedValuesIndex = 0
      return
    self._storedValues.sort()
    ratio = float(self._storedValuesIndex)/float(self._numberOfBins)
    valueIndex = 0
    binIndex = 0
    bin = self.GetNthBin(binIndex)
    border = ratio
    for valueIndex in range(0, self._storedValuesIndex):
      if valueIndex>border:
        binIndex += 1
        bin = self.GetNthBin(binIndex)
        border += ratio
      bin.AddValue(self._storedValues[valueIndex])
    self._storedValuesIndex = 0
